package de.mpg.mpimet.yacgui.xml.interpolation;

/**
 * Copyright (c) 2013 Max-Planck-Institute for Meteorology, Hamburg.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Enumerates all types of available interpolations
 * 
 * @author Maxim Yastremsky
 */

public enum InterpolationMethod {
	
	AVERAGE("average"),
	N_NEAREST_NEIGHBOR("n-nearest_neighbor"),
	CONSERVATIVE("conservative"),
	RADIAL_BASIS_FUNCTION("radial_basis_function"),
	BERNSTEIN_BEZIER("bernstein_bezier"),
	FIXED_VALUE("fixed_value"),
	USER_FILE("user_file"),
	SOURCE2TARGET_MAP("source_to_target_map");
	
	private final String name;
	
	private InterpolationMethod(String s) {
		name = s;
	}
	
	public String getName() {
		return name;
	}

	public boolean equalsName(String otherName) {
		return (otherName == null) ? false : name.equals(otherName);
	}
	
	/**
	 * check the name is in the list of Interpolation method 
	 * @param name String representation of {@link InterpolationMethod} 
	 * @return true if the name is in the list, false if not
	 */
	public static boolean isValidName(String name) {
		if (name == null) return false;

		if (AVERAGE.equalsName(name)
		    || N_NEAREST_NEIGHBOR.equalsName(name)
		    || RADIAL_BASIS_FUNCTION.equalsName(name)
		    || CONSERVATIVE.equalsName(name)
		    || BERNSTEIN_BEZIER.equalsName(name)
		    || FIXED_VALUE.equalsName(name)
		    || USER_FILE.equalsName(name)
		    || SOURCE2TARGET_MAP.equalsName(name))
       		return true;
		return false;
	}
	
	/**
	 * select Interpolation method from name when create Interpolation
	 * @param name String representation of {@link InterpolationMethod} 
	 * @return the String representation of selected InterpolationMethod
	 */
	public static InterpolationMethod fromName(String name) {
		if (name == null) return null;
		
		if (AVERAGE.equalsName(name)) return AVERAGE;
		else if (N_NEAREST_NEIGHBOR.equalsName(name)) return N_NEAREST_NEIGHBOR;
		else if (CONSERVATIVE.equalsName(name)) return CONSERVATIVE;
		else if (BERNSTEIN_BEZIER.equalsName(name)) return BERNSTEIN_BEZIER;
		else if (RADIAL_BASIS_FUNCTION.equalsName(name)) return RADIAL_BASIS_FUNCTION;
		else if (FIXED_VALUE.equalsName(name)) return FIXED_VALUE;
		else if (USER_FILE.equalsName(name)) return USER_FILE;
		else if (SOURCE2TARGET_MAP.equalsName(name)) return SOURCE2TARGET_MAP;
		return null;
	}
	
	@Override
	public String toString() {
		return getName();
	}
}
