/**
 * @file interpolation_enum.c
 *
 * @copyright Copyright  (C)  2013 Rene Redler <rene.redler@mpimet.mpg.de>
 *                                 Teresa Holfeld <teresa.holfeld@zmaw.de>
 *
 * @author Rene Redler <rene.redler@mpimet.mpg.de>
 *         Teresa Holfeld <teresa.holfeld@zmaw.de>
 */
/*
 * Keywords:
 * Maintainer: Rene Redler <rene.redler@mpimet.mpg.de>
 *             Teresa Holfeld <teresa.holfeld@zmaw.de>
 * URL: https://dkrz-sw.gitlab-pages.dkrz.de/yac/
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <string.h>
#include "interpolation_enum.h"

static char average[]               = "average";
static char n_nearest_neighbor[]    = "n-nearest_neighbor";
static char conservative[]          = "conservative";
static char source_to_target_map[]  = "source_to_target_map";
static char fixed_value[]           = "fixed_value";
static char user_file[]             = "user_file";
static char check[]                 = "check";
static char bernstein_bezier[]      = "bernstein_bezier";
static char radial_basis_function[] = "radial_basis_function";

char * yac_interpolation_enum_to_string(const enum yac_interpolation_list interpolation) {
   switch (interpolation) {
   case (AVERAGE):
         return average;
   case (N_NEAREST_NEIGHBOR):
         return n_nearest_neighbor;
   case (CONSERVATIVE):
         return conservative;
   case (SOURCE_TO_TARGET_MAP):
         return source_to_target_map;
   case (FIXED_VALUE):
         return fixed_value;
   case (USER_FILE):
         return user_file;
   case (CHECK):
         return check;
   case (BERNSTEIN_BEZIER):
         return bernstein_bezier;
   case (RADIAL_BASIS_FUNCTION):
         return radial_basis_function;
   default:
      return NULL;
   }
}

char * yac_interpolation_int_to_string(const int interpolation) {

   /* The case selection has to be consistent with what is implemented
    * in interpolation_enum.h . Unfortunately we have no way to check
    * for consistency.
    */

   switch (interpolation) {
   case (AVERAGE):
         return average;
   case (N_NEAREST_NEIGHBOR):
         return n_nearest_neighbor;
   case (CONSERVATIVE):
         return conservative;
   case (SOURCE_TO_TARGET_MAP):
         return source_to_target_map;
   case (FIXED_VALUE):
         return fixed_value;
   case (USER_FILE):
         return user_file;
   case (CHECK):
         return check;
   case (BERNSTEIN_BEZIER):
         return bernstein_bezier;
   case (RADIAL_BASIS_FUNCTION):
         return radial_basis_function;
   default:
      return NULL;
   }
}

enum yac_interpolation_list yac_interpolation_enum_from_string(const char * interpolation) {
   if ( interpolation == NULL ) return UNDEFINED;

   if (strcmp(interpolation, average) == 0) {
      return AVERAGE;
   } else if (strcmp(interpolation, n_nearest_neighbor) == 0) {
      return N_NEAREST_NEIGHBOR;
   } else if (strcmp(interpolation, conservative) == 0) {
      return CONSERVATIVE;
   } else if (strcmp(interpolation, source_to_target_map) == 0) {
      return SOURCE_TO_TARGET_MAP;
   } else if (strcmp(interpolation, fixed_value) == 0) {
      return FIXED_VALUE;
   } else if (strcmp(interpolation, user_file) == 0) {
      return USER_FILE;
   } else if (strcmp(interpolation, check) == 0) {
      return CHECK;
   } else if (strcmp(interpolation, bernstein_bezier) == 0) {
     return BERNSTEIN_BEZIER;
   } else if (strcmp(interpolation, radial_basis_function) == 0) {
     return RADIAL_BASIS_FUNCTION;
   } else {
      return UNDEFINED;
   }
}

char * yac_verified_interpolation(const char * interpolation) {
   if (strcmp(interpolation, average) == 0) {
      return average;
   } else if (strcmp(interpolation, n_nearest_neighbor) == 0) {
      return n_nearest_neighbor;
   } else if (strcmp(interpolation, conservative) == 0) {
      return conservative;
   } else if (strcmp(interpolation, source_to_target_map) == 0) {
      return source_to_target_map;
   } else if (strcmp(interpolation, fixed_value) == 0) {
      return fixed_value;
   } else if (strcmp(interpolation, user_file) == 0) {
      return user_file;
   } else if (strcmp(interpolation, check) == 0) {
      return check;
   } else if (strcmp(interpolation, bernstein_bezier) == 0) {
      return bernstein_bezier;
   } else if (strcmp(interpolation, radial_basis_function) == 0) {
      return radial_basis_function;
   } else {
      return NULL;
   }
}

